#! /bin/bash

cfg_parser ()
{
    ini="$(<$1)"                # read the file
    ini="${ini//[/\[}"          # escape [
    ini="${ini//]/\]}"          # escape ]
    IFS=$'\n' && ini=( ${ini} ) # convert to line-array
    ini=( ${ini[*]//;*/} )      # remove comments with ;
    ini=( ${ini[*]/\    =/=} )  # remove tabs before =
    ini=( ${ini[*]/=\   /=} )   # remove tabs be =
    ini=( ${ini[*]/\ =\ /=} )   # remove anything with a space around =
    ini=( ${ini[*]/#\\[/\}$'\n'cfg.section.} ) # set section prefix
    ini=( ${ini[*]/%\\]/ \(} )    # convert text2function (1)
    ini=( ${ini[*]/=/=\( } )    # convert item to array
    ini=( ${ini[*]/%/ \)} )     # close array parenthesis
    ini=( ${ini[*]/%\\ \)/ \\} ) # the multiline trick
    ini=( ${ini[*]/%\( \)/\(\) \{} ) # convert text2function (2)
    ini=( ${ini[*]/%\} \)/\}} ) # remove extra parenthesis
    ini[0]="" # remove first element
    ini[${#ini[*]} + 1]='}'    # add the last brace
    eval "$(echo "${ini[*]}")" # eval the result
}

APPSDIR="/var/www/apps/roundcube"
RCNUMBER="0.8.5"
RCPACKED="roundcubemail-"$RCNUMBER".tar.gz"
RCUNPACKED="roundcubemail-"$RCNUMBER
cfg_parser "app.ini"

#Set mysql values
cfg.section.mysql
DBUSER=$database_user
DBPASS=$database_password
DBNAME=$database_name
DBHOST=$database_host

#Set soap values
cfg.section.remote_api
SOAPHOST=$remote_hostname
SOAPUSER=$remote_user
SOAPPASS=$remote_password

wget "http://downloads.sourceforge.net/project/roundcubemail/roundcubemail/"$RCNUMBER"/"$RCPACKED

if [ -e $RCPACKED ]; then
  mkdir $APPSDIR
  tar xfvz $RCPACKED

  if [ $(mysql -u$DBUSER -p$DBPASS -N -s -e "select count(*) from information_schema.tables where table_schema='$DBNAME' and table_name='users';") -eq 1 ]; then
    mysql -u$DBUSER -p$DBPASS $DBNAME < $RCUNPACKED/SQL/mysql.update.sql
  else
    mysql -u$DBUSER -p$DBPASS $DBNAME < $RCUNPACKED/SQL/mysql.initial.sql
  fi
  
  mkdir $APPSDIR
  mv -f $RCUNPACKED/* $APPSDIR/
  mv -f $RCUNPACKED"/.htaccess" $APPSDIR/
  rm -rf $RCPACKED $APPSDIR"/installer" $APPSDIR"/CHANGELOG" $APPSDIR"/LICENSE" $APPSDIR"/README.md" $APPSDIR"/INSTALL" $APPSDIR"/UPGRADING" $APPSDIR"/SQL"
  
  #svn export "http://svn.web-wack.at/svn/ispconfig3_roundcube/trunk/" $APPSDIR"/plugins/"
  mv "plugins/"* $APPSDIR"/plugins/"
  
  chown -R ispapps:ispapps $APPSDIR
  chmod 755 $APPSDIR"/temp" $APPSDIR"/logs"
  
  #Create Roundcube DBconfigfile
  sed "25 s/.*/\$rcmail_config['db_dsnw'] = 'mysqli:\/\/$DBUSER:$DBPASS@$DBHOST\/$DBNAME';/" $APPSDIR"/config/db.inc.php.dist" > $APPSDIR"/config/db.inc.php"
  
  #Create Roundcube Mainconfigfile
  sed "143  s/.*/\$rcmail_config['smtp_server'] = \"\%h\";/" $APPSDIR"/config/main.inc.php.dist" > $APPSDIR"/config/main.inc.php"
  sed "151  s/.*/\$rcmail_config['smtp_user'] = \"\%u\";/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "155  s/.*/\$rcmail_config['smtp_pass'] = \"\%p\";/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "379 s/.*/\$rcmail_config[\'plugins\'] = array(\"jqueryui\",\"ispconfig3_account\", \"ispconfig3_autoreply\", \"ispconfig3_autoselect\", \"ispconfig3_fetchmail\", \"ispconfig3_filter\", \"ispconfig3_forward\", \"ispconfig3_pass\", \"ispconfig3_spam\", \"ispconfig3_wblist\");/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "443  s/.*/\$rcmail_config['create_default_folders'] = TRUE;/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "449 s/.*/\$rcmail_config['quota_zero_as_unlimited'] = TRUE;/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "714 s/.*/\$rcmail_config['show_images'] = 1;/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  sed "727 s/.*/\$rcmail_config['preview_pane'] = TRUE;/" $APPSDIR"/config/main.inc.php" > $APPSDIR"/config/main.inc.php.tmp"
  mv -f $APPSDIR"/config/main.inc.php.tmp" $APPSDIR"/config/main.inc.php"
  
  #Create ISPConfig Plugins Configfile
  sed "3 s/.*/\$rcmail_config['remote_soap_user'] = '$SOAPUSER';/" $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php.dist" > $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php"
  sed "4 s/.*/\$rcmail_config['remote_soap_pass'] = '$SOAPPASS';/" $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php" > $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php.tmp"
  mv -f $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php.tmp" $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php"
  sed "5 s/.*/\$rcmail_config['soap_url'] = 'http:\/\/$SOAPHOST\/remote\/';/" $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php" > $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php.tmp"
  mv -f $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php.tmp" $APPSDIR"/plugins/ispconfig3_account/config/config.inc.php"
  
  echo "[OK]"
fi